package Grammar;

use 5.008004;
use strict;
use warnings;
use Parse::Yapp;

require Exporter;

our @ISA = qw(Exporter);

our @EXPORT = qw( );

our $VERSION = '0.02';


sub Parse {
  my $grammar = shift;

  my $x = new Parse::Yapp::Parse;
  my $r = $x->Parse($grammar);

  # Supress not used keys
  delete @$r{'EXPECT', 'HEAD', 'TAIL', 'PREC'};


  my $rules = $r->{RULES};
  shift @$rules; # supress super-rule

  # recompute production indices for each syntax variable
  my @vars = keys(%{$r->{NTERM}});
  for my $var (@vars) {
    my $lines = $r->{NTERM}{$var};
    @$lines = map { --$_ } @$lines;
  }

  #supress priority and code
  for my $rule (@$rules) {
   $#$rule = 1;
  }
  #supress token priorities
  my @tokens = keys(%{$r->{TERM}});
  $r->{TERM} = \@tokens;
  return $r;
}

1;
__END__

=head1 NAME

Grammar - Reads a yacc or yapp grammar and returns a reference to a hash 
          describing the grammar

=head1 SYNOPSIS

  use Grammar;
  my $x = Grammar::Parse($grammar);

=head1 DESCRIPTION

This module is written only for educational purposes for
the subject "Procesadores de Lenguajes" at ETSII, Universidad 
de La Laguna.

Function Grammar::Parse receives as input a string
containing the grammar and returns a reference to 
a hash describing the grammar.
The input grammar follows the rules of a yacc grammar.
However code is ignored.

Follows an example. See the directory scripts for more
examples:

    $ cat -n use.pl
         1  #!/usr/bin/perl -w -I../lib
         2  use strict;
         3  use Grammar;
         4  use Data::Dumper;
         5
         6  my $grammar = <<'EOG';
         7  %start A
         8  %%
         9  A : 'a' A 'b' | A B
        10  ;
        11  B :  | 'c' B
        12  ;
        13  %%
        14  EOG
        15
        16  my $x = Grammar::Parse($grammar);

After line 16 variable $x contains a hash with the following keys:

          'SYMS' => { # list of symbols: tokens and variables
                      'A' => 3,    # variable => line number
                      '\'c\'' => 5,# token    => line number
                      '\'b\'' => 3,
                      '\'a\'' => 3,
                      'B' => 5
                    },
          'NULL' => { 'B' => 1 }, # nullable variables
          'RULES' => [
                       [ 'A', [ '\'a\'', 'A', '\'b\'' ] ], # 1st rule for A
                       [ 'A', [ 'A', 'B' ] ],
                       [ 'B', [] ],
                       [ 'B', [ '\'c\'', 'B' ] ]
                     ],
          'START' => 'A', # grammar start symbols
          'TERM' => [ '\'c\'', '\'b\'', '\'a\'' ], # token list
          'NTERM' => {
                       'A' => [ 0, 1 ], # indices of A productions
                       'B' => [ 2, 3 ]  # indices of B productions
                     }
        };


=head2 EXPORT

Nothing

=head1 DEPENDENCIES

The module depends on Parse::Yapp

=head1 SEE ALSO

http://nereida.deioc.ull.es/~lhp/perlexamples/ 

=head1 AUTHOR

Casiano Rodriguez Leon

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by Casiano Rodriguez Leon

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.


=cut
